/***************************************************
 * Example code for the ISRemote API               *
 * compile with Turbo C, Borland C/C++ or DJGPP    *
 * written by Sebastian Zeller                     *
 ***************************************************/

#include <stdio.h>
#include <dos.h>

#ifdef __DJGPP__
#include <dpmi.h>
#define INT21 0x21
#define FARPTR
#else
#define FARPTR far
#endif

#define magic_id 0xEA90                          /* the id to look for */
#define min_IRQ 2                                /* search starts at min_IRQ + 1 */
#define max_IRQ 7                                /* search ends at max_IRQ */

typedef struct {
		unsigned short 	ISR_id;          /* the id to test */
		unsigned short  ISR_reserved_o;  /* reserved */
		unsigned short  ISR_reserved_s;  /* reserved */
		unsigned short  IR_tick;         /* tick count of the last received code, use it
						    to find out if you've got a new command */
		unsigned short  IR_address;      /* address part of the received IR code */
		unsigned short	IR_command;      /* command part of the received IR code */
		unsigned short  ISR_version;     /* ISRemote's version number x 100 */
		unsigned short  MHz_speed;       /* in RDTSC mode: MHz speed of your computer
						    otherwise: 0 */
		unsigned short  Port_address;    /* used LPT/COM port address */
		unsigned short  IRQ_number;      /* used IRQ */
	       } ISR_api;                        /* don't alter any of the above values ! */


int main(void)
{
	ISR_api FARPTR * ISR_pointer;
	int IRQ = min_IRQ;
        unsigned short IR_oldtick;
#ifdef __DJGPP__
	__dpmi_regs r;
	ISR_pointer = (ISR_api *) malloc(sizeof(ISR_api));
#endif

	while (ISR_pointer->ISR_id != magic_id  &&  IRQ < max_IRQ)     /* loop through the IRQs to test */
#ifdef __DJGPP__
        {
                r.x.ax = ++IRQ + 0x3508;
                __dpmi_int(INT21, &r);
                movedata(_go32_conventional_mem_selector(), (r.x.es << 4)     /* get interrupt vector */
                         + r.x.bx, _my_ds(), ISR_pointer, sizeof(ISR_api));   /* copy the API data block to our structure */
        }
#else
		ISR_pointer = (ISR_api far *) getvect(++IRQ + 8);      /* assign pointer of the IRQ's ISR to our structure */
#endif

	if (ISR_pointer->ISR_id != magic_id)
	{
		printf("\nISRemote not found !\n");                    /* oops - not found */
                return 1;
        }

        printf("\nISRemote version %.2f found.\n",                     /* gotcha */
		(float) ISR_pointer->ISR_version / 100);

        IR_oldtick = ISR_pointer->IR_tick;

        while (!kbhit())
        {
#ifdef __DJGPP__
              movedata(_go32_conventional_mem_selector(), (r.x.es << 4)    /* copy data block again */
                       + r.x.bx, _my_ds(), ISR_pointer, sizeof(ISR_api));
#endif
              if (ISR_pointer->IR_tick != IR_oldtick)                  /* test for new command */
              {
                   printf("Received NECIR command %X %X.\n",
                           ISR_pointer->IR_address, ISR_pointer->IR_command);
                   IR_oldtick = ISR_pointer->IR_tick;
              }
        }

#ifdef __DJGPP__
        free(ISR_pointer);
#endif

        return 0;
}
